@@ -353,13 +353,10 @@ QoreHashNode* MyResult::getSingleRow(ExceptionSink* xsink) {
 }
 
 // returns a hash of empty lists keyed by column name
-QoreHashNode* MyResult::setupColumns() {
-   QoreHashNode* h = new QoreHashNode;
+void MyResult::setupColumns(QoreHashNode& h) {
    QoreString tstr;
    for (int i = 0; i < num_fields; i++)
-      assign_column_value(tstr, enc, field[i].name, *h, new QoreListNode);
-
-   return h;
+      assign_column_value(tstr, enc, field[i].name, h, new QoreListNode);
 }
 
 void MyResult::bind(MYSQL_STMT *stmt) {
@@ -885,12 +882,18 @@ int QoreMysqlBindGroup::getDataRows(QoreListNode& l, ExceptionSink* xsink, int m
    return 0;
 }
 
-int QoreMysqlBindGroup::getDataColumns(QoreHashNode& h, ExceptionSink* xsink, int max) {
-   assert(!h.empty());
+int QoreMysqlBindGroup::getDataColumns(QoreHashNode& h, ExceptionSink* xsink, int max, bool cols) {
+   assert(h.empty());
+
+   if (cols)
+      myres.setupColumns(h);
 
    // row count
    int c = 0;
    while ((max < 0 || c < max) && !mysql_stmt_fetch(stmt)) {
+      if (h.empty())
+         myres.setupColumns(h);
+
       HashIterator hi(h);
       int i = 0;
       while (hi.next()) {
@@ -903,19 +906,19 @@ int QoreMysqlBindGroup::getDataColumns(QoreHashNode& h, ExceptionSink* xsink, in
    return 0;
 }
 
-AbstractQoreNode* QoreMysqlBindGroup::exec(ExceptionSink* xsink) {
+AbstractQoreNode* QoreMysqlBindGroup::exec(ExceptionSink* xsink, bool cols) {
    if (execIntern(xsink))
       return 0;
 
    if (myres) {
-      ReferenceHolder<QoreHashNode> h(myres.setupColumns(), xsink);
+      ReferenceHolder<QoreHashNode> h(new QoreHashNode, xsink);
 
       if (!mysql_stmt_affected_rows(stmt))
          return h.release();
 
       myres.bind(stmt);
 
-      return getDataColumns(**h, xsink) ? 0 : h.release();
+      return getDataColumns(**h, xsink, -1, cols) ? 0 : h.release();
    }
 
    return !hasOutput
@@ -965,7 +968,7 @@ QoreHashNode* QoreMysqlBindGroup::selectRow(ExceptionSink* xsink) {
 
          // see if there is a second row
          if (!mysql_stmt_fetch(stmt)) {
-            xsink->raiseException("MYSQL-SELECT-ROW-ERROR", "SQL passed to selectRow() returned more than 1 row");
+            xsink->raiseException("DBI-SELECT-ROW-ERROR", "SQL passed to selectRow() returned more than 1 row");
             return 0;
          }
 
@@ -1117,7 +1120,7 @@ QoreListNode* QoreMysqlPreparedStatement::fetchRows(int rows, ExceptionSink *xsi
 }
 
 QoreHashNode* QoreMysqlPreparedStatement::fetchColumns(int rows, ExceptionSink *xsink) {
-   ReferenceHolder<QoreHashNode> h(myres.setupColumns(), xsink);
+   ReferenceHolder<QoreHashNode> h(new QoreHashNode, xsink);
    return !getDataColumns(**h, xsink) ? h.release() : 0;
 }
 
@@ -1448,7 +1451,7 @@ static QoreHashNode* get_result_set(const QoreMysqlConnection& conn, MYSQL_RES *
       }
       rn++;
       if (single_row && rn > 1) {
-         xsink->raiseException("MYSQL-SELECT-ROW-ERROR", "SQL passed to selectRow() returned more than 1 row");
+         xsink->raiseException("DBI-SELECT-ROW-ERROR", "SQL passed to selectRow() returned more than 1 row");
          return 0;
       }
       for (int i = 0; i < num_fields; i++) {
@@ -1596,7 +1599,7 @@ static AbstractQoreNode* qore_mysql_select(Datasource *ds, const QoreString* qst
    if (rc == 1)
       return qore_mysql_do_sql(conn, qstr, args, xsink);
 
-   return bg.exec(xsink);
+   return bg.exec(xsink, true);
 #else
    return qore_mysql_do_sql(conn, qstr, args, xsink);
 #endif
